use std::num::NonZeroU8;
use std::path::Path;

use anyhow::Context;
use axum::http::HeaderValue;
use ipnet::IpNet;
use serde::Deserialize;
use serde::Deserializer;
use service_binding::Binding;
use tower_http::cors::AllowOrigin;

#[derive(Debug, Deserialize)]
pub struct Server {
    /// The address to bind to. The syntax is described at:
    /// <https://github.com/wiktor-k/service-binding/blob/main/README.md#supported-schemes>
    #[serde(deserialize_with = "deserialize_binding")]
    pub bind_address: Binding,

    /// The log level
    #[serde(deserialize_with = "deserialize_level", default = "default_level")]
    pub log_level: tracing::Level,

    /// The origins to allow for CORS. In the configuration, this is a list (of URLs).
    #[serde(deserialize_with = "deserialize_allow_origin")]
    pub cors_origins: AllowOrigin,
}

#[derive(Debug, Deserialize, Default)]
pub struct SectionFilters {
    /// If set, only users with any of the specified groups will be able to access the section
    pub groups: Option<Vec<String>>,

    /// If set, only users with an IP in any specified IP nets will be able to access the section
    pub ips: Option<Vec<IpNet>>,

    /// If set, only users accessing this server with any of the specified hostnames will be able to access the section
    pub hosts: Option<Vec<String>>,
}

#[derive(Debug, Deserialize)]
pub struct PasswordProtection {
    /// Hint to display to the user to help him guess the password
    pub hint: String,

    /// The password the user will need to input to view the section
    pub password: String,
}

#[derive(Debug, Deserialize)]
pub struct Section {
    /// URL where the user can access the icon
    pub icon: String,

    /// URL where the user can be redirected to
    pub href: String,

    /// Title of the section
    pub title: String,

    /// The section description
    pub descr: String,

    /// Filters to restrict who can access this section.
    /// If multiple fields are set, the user must verify all of them.
    #[serde(default)]
    pub filters: SectionFilters,

    /// If multiple sections have the same title, the client may display only the one
    /// with the highest priority
    pub priority: Option<NonZeroU8>,

    /// If set, the section will need a password to be displayed
    pub protection: Option<PasswordProtection>,
}

#[derive(Debug, Deserialize)]
pub struct Config {
    pub server: Server,
    pub sections: Vec<Section>,
}

/// Parse the configuration.
///
/// Configuration may come from the following sources:
/// - A JSON file: The file is structured in the same way as the [Config] struct
///
/// Sources are read and merged in the above order, the latter sources
/// taking overriding the previous ones in case of conflict.
pub fn parse_config(config_json: &Path) -> anyhow::Result<Config> {
    let file_content =
        std::fs::read_to_string(config_json).context("Error while reading the file")?;

    serde_json::from_str(&file_content).context("Error while parsing the configuration file")
}

fn deserialize_binding<'de, D>(deserializer: D) -> Result<Binding, D::Error>
where
    D: Deserializer<'de>,
{
    let value: String = Deserialize::deserialize(deserializer)?;

    value.parse().map_err(serde::de::Error::custom)
}

fn deserialize_level<'de, D>(deserializer: D) -> Result<tracing::Level, D::Error>
where
    D: Deserializer<'de>,
{
    let value: String = Deserialize::deserialize(deserializer)?;

    value.parse().map_err(serde::de::Error::custom)
}

const fn default_level() -> tracing::Level {
    tracing::Level::INFO
}

fn deserialize_allow_origin<'de, D>(deserializer: D) -> Result<AllowOrigin, D::Error>
where
    D: Deserializer<'de>,
{
    let values: Vec<String> = Deserialize::deserialize(deserializer)?;

    let values: Vec<HeaderValue> = values
        .into_iter()
        .map(|s| s.parse::<HeaderValue>())
        .collect::<Result<Vec<_>, _>>()
        .map_err(serde::de::Error::custom)?;

    Ok(AllowOrigin::from(values))
}
