use axum::Router;
use axum::routing::get;

use crate::state::SharedState;

pub fn router() -> Router<SharedState> {
    Router::new().nest(
        "/dashboard",
        Router::new().route("/sections", get(get::sections)),
    )
}

mod get {
    use std::num::NonZeroU8;

    use anyhow::Context;
    use axum::Json;
    use axum::extract::State;
    use axum::http::HeaderMap;
    use base64::Engine;
    use base64::prelude::BASE64_STANDARD;
    use serde::Serialize;
    use tracing::debug;

    use crate::config::PasswordProtection;
    use crate::state::SharedState;
    use crate::utils::CustomIterator as _;
    use crate::utils::RouteResult;
    use crate::utils::get_request_host;
    use crate::utils::get_user_groups;
    use crate::utils::get_user_ip;

    #[derive(Debug, Serialize)]
    pub struct VisibleSection {
        /// Path where the icon is served
        icon: String,

        /// URL where to link the section to
        href: String,

        /// Title of the section
        title: String,

        /// The section description
        descr: String,

        /// Priority if multiple sections have the same title (higher is more priority)
        priority: Option<NonZeroU8>,
    }

    #[derive(Debug, Serialize)]
    pub struct PasswordProtectedSection {
        /// Hint displayed to the user
        hint: String,

        /// Base64-encoded password-protected data
        data: String,
    }

    #[derive(Debug, Serialize)]
    #[serde(untagged)]
    pub enum Section {
        Visible(VisibleSection),
        PasswordProtected(PasswordProtectedSection),
    }

    #[axum::debug_handler]
    /// Get the sections the user has access to
    pub async fn sections(
        state: State<SharedState>,
        headers: HeaderMap,
    ) -> RouteResult<Json<Vec<Section>>> {
        debug!("The request headers are: {:?}", headers);

        // Get the user groups from the Trusted SSO header
        let user_groups: Vec<&str> = get_user_groups(&headers);
        debug!("User has the following groups: {:?}", user_groups);

        // Get the user IP address
        let user_ip =
            get_user_ip(&headers).context("Could not get user IP from request headers")?;
        debug!("User has the following IP: {:?}", user_ip);

        // Get the request host
        let request_host = get_request_host(&headers).context("Could not get the request Host")?;
        debug!("User accessed this server via the following host: {request_host}");

        let allowed_sections = state
            .sections
            .iter()
            // Filter on request host
            .filter_or(
                |section| {
                    section
                        .filters
                        .hosts
                        .as_ref()
                        .is_none_or(|hosts| hosts.iter().any(|host| host == request_host))
                },
                |section| {
                    debug!(
                        "Section {} has not passed the 'hosts' filter",
                        section.title
                    );
                },
            )
            // Filter on user IP
            .filter_or(
                |section| {
                    section.filters.ips.as_ref().is_none_or(|ip_nets| {
                        ip_nets.iter().any(|ip_net| ip_net.contains(&user_ip))
                    })
                },
                |section| debug!("Section {} has not passed the 'IPs' filter", section.title),
            )
            // Filter on user groups
            .filter_or(
                |section| {
                    section.filters.groups.as_ref().is_none_or(|groups| {
                        groups
                            .iter()
                            .any(|group| user_groups.contains(&group.as_str()))
                    })
                },
                |section| {
                    debug!(
                        "Section {} has not passed the 'groups' filter",
                        section.title
                    );
                },
            );

        // Collect into serializable sections
        let allowed_sections: Vec<Section> = allowed_sections
            .map(|section| {
                // Create the section
                let visible_section = VisibleSection {
                    icon: section.icon.to_string(),
                    href: section.href.to_string(),
                    title: section.title.to_string(),
                    descr: section.descr.to_string(),
                    priority: section.priority,
                };

                // If there is password protection to be done, we need to apply it
                // FIXME: This could be done once at startup
                match &section.protection {
                    Some(protection) => {
                        Section::PasswordProtected(protect_section(visible_section, protection))
                    }
                    None => Section::Visible(visible_section),
                }
            })
            .collect();
        // TODO
        // debug!(
        //     "User has access to the following public sections: {:?}",
        //     allowed_sections
        //         .iter()
        //         .map(|s| &s.title)
        //         .collect::<Vec<_>>()
        // );

        // Return the sections
        Ok(Json(allowed_sections))
    }

    /// FIXME: This is not secure. However, this is fine for now (since this is just
    ///   to obfuscate sections to the lambda user).
    #[derive(Debug, Serialize)]
    struct NotSecureEncryptedData {
        password: String,
        section: VisibleSection,
    }

    /// Encode the section using the same password-protection algorithm as the frontend.
    fn protect_section(
        visible_section: VisibleSection,
        protection: &PasswordProtection,
    ) -> PasswordProtectedSection {
        // "Encrypt" the data
        let encrypted_data = NotSecureEncryptedData {
            password: protection.password.clone(),
            section: visible_section,
        };

        // Convert to JSON
        let json_str = serde_json::to_string(&encrypted_data)
            .expect("Error while password-protecting section: Could not encode to JSON");

        // Encode as base64
        let base64_str = BASE64_STANDARD.encode(json_str);

        PasswordProtectedSection {
            hint: protection.hint.clone(),
            data: base64_str,
        }
    }
}
