use std::path::PathBuf;

use anyhow::Context;
use axum::Router;
use axum::http::Method;
use state::State;
use tokio::net::TcpListener;
use tokio::net::UnixListener;
use tower::ServiceBuilder;
use tower_http::cors::CorsLayer;
use tower_http::trace::TraceLayer;

mod config;
mod dashboard;
mod state;
mod utils;

#[tokio::main(flavor = "current_thread")]
async fn main() -> anyhow::Result<()> {
    let config_json: PathBuf = std::env::args()
        .nth(1)
        .context("Expected JSON config file as argument")?
        .into();
    let config = config::parse_config(&config_json)?;

    tracing_subscriber::fmt()
        .with_max_level(config.server.log_level)
        .init();

    let binding = config.server.bind_address;
    let cors_layer = CorsLayer::new()
        .allow_methods(Method::GET)
        .allow_origin(config.server.cors_origins);

    let state = State::new_shared(config.sections);

    let app = Router::new()
        .merge(dashboard::router())
        .layer(
            ServiceBuilder::new()
                .layer(TraceLayer::new_for_http())
                .layer(cors_layer),
        )
        .with_state(state);

    match binding
        .try_into()
        .context("Could not get a listener from the binding address")?
    {
        service_binding::Listener::Unix(listener) => {
            axum::serve(
                UnixListener::from_std(listener).unwrap(),
                app.into_make_service(),
            )
            .await?;
        }

        service_binding::Listener::Tcp(listener) => {
            axum::serve(
                TcpListener::from_std(listener).unwrap(),
                app.into_make_service(),
            )
            .await?;
        }

        service_binding::Listener::NamedPipe(_) => {
            anyhow::bail!("Named pipes are not supported")
        }
    }

    Ok(())
}
