use std::iter::Filter;
use std::net::IpAddr;

use axum::http::HeaderMap;
use axum::http::StatusCode;
use axum::response::IntoResponse;
use axum::response::Response;
use tracing::debug;

pub type RouteResult<T> = Result<T, RouteError>;

pub struct RouteError(anyhow::Error);

impl IntoResponse for RouteError {
    fn into_response(self) -> Response {
        (
            StatusCode::INTERNAL_SERVER_ERROR,
            format!("Something went wrong: {}", self.0),
        )
            .into_response()
    }
}

impl<E> From<E> for RouteError
where
    E: Into<anyhow::Error>,
{
    fn from(err: E) -> Self {
        Self(err.into())
    }
}

/// Get the user groups from the request headers.
/// This supposes that we are behind a reverse-proxy that forwards authentication.
pub fn get_user_groups(headers: &HeaderMap) -> Vec<&str> {
    let header_groups = headers.get("Remote-Groups");
    debug!(
        "Trusted SSO header found? {}",
        if header_groups.is_some() { "Yes" } else { "No" }
    );
    header_groups
        .and_then(|val| val.to_str().ok())
        .map(|val| val.split(',').collect())
        .unwrap_or_default()
}

/// Get the user IP from the request headers.
/// This supposes that we are behind a reverse-proxy that sets this header.
pub fn get_user_ip(headers: &HeaderMap) -> Option<IpAddr> {
    let header_forwarded_for = headers.get("X-Forwarded-For");
    debug!(
        "X-Forwarded-For header found? {}",
        if header_forwarded_for.is_some() {
            "Yes"
        } else {
            "No"
        }
    );
    header_forwarded_for
        .and_then(|val| val.to_str().ok())
        // The header may have multiple IPs. The client IP is the first one
        .and_then(|val| val.split(',').next())
        .and_then(|val| val.parse::<IpAddr>().ok())
}

/// Get the host requested by the user
pub fn get_request_host(headers: &HeaderMap) -> Option<&str> {
    let header_forwarded_host = headers.get("X-Forwarded-Host");
    debug!(
        "X-Forwarded-Host header found? {}",
        if header_forwarded_host.is_some() {
            "Yes"
        } else {
            "No"
        }
    );
    header_forwarded_host.and_then(|val| val.to_str().ok())
}

pub trait CustomIterator: Iterator {
    /// Same as [`Iterator::filter`], but call another function for filtered-out elements
    fn filter_or<P, F>(
        self,
        mut predicate: P,
        mut handle_out: F,
    ) -> Filter<Self, impl FnMut(&Self::Item) -> bool>
    where
        Self: Sized,
        P: FnMut(&Self::Item) -> bool,
        F: FnMut(&Self::Item),
    {
        // self.filter(predicate)
        self.filter(move |e| {
            if predicate(e) {
                true
            } else {
                handle_out(e);
                false
            }
        })
    }
}

impl<T: Iterator> CustomIterator for T {}
