import type { SectionProtection } from "@src/types/config";

/** Contents of a section. Can be used to create a card in the dashboard */
export interface Section {
  icon: string;
  href: string;
  title: string;
  descr: string;
  priority?: number;
}

/** A password-protected section. The contents must be decoded before usage */
export interface EncryptedSection {
  /** A hint to display to the user to help him guess the password */
  hint: string;

  /** Base64-encoded {@link NotSecureEncryptedData} */
  data: string;
}

export const isEncryptedSection = (
  s: Section | EncryptedSection
): s is EncryptedSection => (s as EncryptedSection).hint !== undefined;
export const isSection = (s: Section | EncryptedSection): s is Section =>
  !isEncryptedSection(s);

// #region Card creation
/** Create the HTML element for a card */
export const createCard = ({
  icon,
  href,
  title,
  descr,
}: Section): HTMLLIElement => {
  const img = document.createElement("img");
  img.src = icon;
  img.alt = "logo";

  const h3 = document.createElement("h3");
  const a = document.createElement("a");
  a.href = href;
  a.textContent = title;
  h3.appendChild(a);

  const p = document.createElement("p");
  p.textContent = descr;

  const li = document.createElement("li");
  li.classList.add("card");
  li.append(img, h3, p);

  return li;
};

/** Create the HTML element for a password-protected card */
export const createEncryptedCard = ({
  hint,
  data,
}: EncryptedSection): HTMLLIElement => {
  const legend = document.createElement("legend");
  legend.textContent = hint;

  const dataElt = document.createElement("input");
  dataElt.name = "data";
  dataElt.type = "hidden";
  dataElt.value = data;

  const passwordInput = document.createElement("input");
  passwordInput.name = "password";
  passwordInput.type = "password";
  passwordInput.required = true;

  const passwordLabel = document.createElement("label");
  passwordLabel.textContent = "Password:";
  passwordLabel.append(passwordInput);

  const decryptButton = document.createElement("button");
  decryptButton.textContent = "Decrypt";

  const form = document.createElement("form");
  form.append(legend, dataElt, passwordLabel, decryptButton);

  const li = document.createElement("li");
  li.classList.add("card", "encrypted");
  li.append(form);

  return li;
};
// #endregion

// #region Crypto
// FIXME: This is not secure. However, this is fine for now (since this is just
//   to obfuscate sections to the lambda user).
interface NotSecureEncryptedData {
  password: string;
  section: Section;
}

/** Encrypt a section */
export const encryptSection = async (
  section: Section,
  { hint, password }: SectionProtection
): Promise<EncryptedSection> => {
  const encryptedData: NotSecureEncryptedData = { password, section };
  const encryptedDataStr = JSON.stringify(encryptedData);
  const data = btoa(encryptedDataStr);

  return { hint, data };
};

/** Try to decrypt a section using the given password. If the decryption fails, return null */
export const tryDecryptSection = async (
  encodedEncryptedData: string,
  password: string
): Promise<Section | null> => {
  const encryptedDataStr = atob(encodedEncryptedData);
  const encryptedData: NotSecureEncryptedData = JSON.parse(encryptedDataStr);

  if (password !== encryptedData.password) {
    return null;
  }
  return encryptedData.section;
};
// #endregion
